@@ -21,11 +21,21 @@
 
 /* Changelog Phytec:
     2015-09-01: Stefan Christ <s.christ@phytec.de>
-	- copy base file from src/egl/opengles2/es2gears.c in mesa-demos-8.2.0 
+        - copy base file from src/egl/opengles2/es2gears.c in mesa-demos-8.2.0
           (License is MIT)
-        - add i.MX6 and EGL initialization to main function 
+        - add i.MX6 and EGL initialization to main function
         - replace and remove some eglut stuff
         - add signal handler to exit on CTRL+C
+    2016-06-09: Stefan Christ <s.christ@phytec.de>
+        - Mask i.MX6 specific code in #ifdef's
+        - Cleanup some whitespaces
+    2016-07-11: Stefan Christ <s.christ@phytec.de>
+        - Initialize eglSwapInterval to use double buffering.
+          NOTE: For i.MX6/Freescale/NXP OpenGL libraries you must set the
+          environment variable FB_MULTI_BUFFER, too. Example:
+              $ FB_MULTI_BUFFER=2 es2gears
+    2016-08-01: Wadim Egorov <w.egorov@phytec.de>
+        - use GL_VIEWPORT to set the width and height of gears
 */
 
 /*
@@ -730,6 +740,7 @@ void term(int signum)
    quit = 1;
 }
 
+#if defined(LINUX) && defined(EGL_API_FB)
 void print_native_info(EGLNativeDisplayType native_display, EGLNativeWindowType native_window)
 {
    int height, width, stride, bitsperpixel, x, y;
@@ -744,19 +755,27 @@ void print_native_info(EGLNativeDisplayType native_display, EGLNativeWindowType
    printf("native_window: x %d y %d width %d height %d bitsperpixel %d offset %u\n",
           x, y, width, height, bitsperpixel, offset);
 }
+#endif
 
 
 int
 main(int argc, char *argv[])
 {
    int ret;
+   GLint vp[4];
 
+#if defined(LINUX) && defined(EGL_API_FB)
    /* i.MX6 specific framebuffer initialization */
-   int fdnumber = 0; 
+   int fdnumber = 0;
    EGLNativeDisplayType native_display = fbGetDisplayByIndex(fdnumber);
    EGLNativeWindowType native_window = fbCreateWindow(native_display, 0, 0, 0, 0); // maximal values
 
    print_native_info(native_display, native_window);
+#else
+   /* Default initializied copied from Qt sourcecode */
+   EGLNativeDisplayType native_display = EGL_DEFAULT_DISPLAY;
+   EGLNativeWindowType native_window = 0;
+#endif
 
    /* generic EGL initialization */
    EGLDisplay egl_display = eglGetDisplay(native_display);
@@ -821,6 +840,16 @@ main(int argc, char *argv[])
    /* associate the egl-context with the egl-surface */
    eglMakeCurrent(egl_display, egl_surface, egl_surface, egl_context);
 
+   /* Enable display synchronisation and buffering
+    *    0 - no sync and no buffering
+    *    1 - sync and double buffering
+    *    2 - sync and triple buffering
+    */
+   if (eglSwapInterval(egl_display, 1) != EGL_TRUE) {
+      fprintf(stderr, "Unable to initialize eglSwapInterval (eglError: %d)\n", eglGetError());
+      return 1;
+   }
+
    /* Setting up SIGTERM, SIGINT and SIGQUIT handler */
    struct sigaction action;
    memset(&action, 0, sizeof(struct sigaction));
@@ -833,7 +862,14 @@ main(int argc, char *argv[])
    gears_init();
 
    int width, height;
-   fbGetDisplayGeometry(native_display, &width, &height); /* i.MX6 sepcific */
+#if defined(LINUX) && defined(EGL_API_FB)
+   /* i.MX6 specific */
+   fbGetDisplayGeometry(native_display, &width, &height);
+#else
+   glGetIntegerv(GL_VIEWPORT, vp);
+   height = vp[3];
+   width = vp[2];
+#endif
    gears_reshape(width, height);
 
    /* Simple mainloop without keyboard events */
@@ -852,9 +888,11 @@ main(int argc, char *argv[])
    eglDestroySurface(egl_display, egl_surface);
    eglTerminate(egl_display);
 
+#if defined(LINUX) && defined(EGL_API_FB)
    /* i.MX6 specific framebuffer cleanup */
    fbDestroyWindow(native_window);
    fbDestroyDisplay(native_display);
+#endif
 
    return 0;
 }
